@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2014 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2015 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -118,7 +118,7 @@ bool MyApp::OnInit(void)
 {//=====================
 
 int j;
-wxChar *p;
+const wxChar *p;
 char param[120];
 
 
@@ -642,7 +642,7 @@ void OnOptions2(int event_id)
 		break;
 
 	case MENU_PATH_DATA:
-		string = wxDirSelector(_T("espeak_data directory"), wxEmptyString);
+		string = wxDirSelector(_T("espeak_data directory"), path_espeakdata);
 		if(!string.IsEmpty())
 		{
 			if(!wxDirExists(string+_T("/voices")))
@@ -744,7 +744,7 @@ void MyFrame::OnTools(wxCommandEvent& event)
 	int debug_flag=0;
 	char fname_log[sizeof(path_dsource)+12];
 	char err_fname[sizeof(path_home)+15];
-	static const char utf8_bom[] = {0xef,0xbb,0xbf,0};
+	static unsigned const char utf8_bom[] = {0xef,0xbb,0xbf,0};
 
 	switch(event.GetId())
 	{
