@@ -135,7 +135,7 @@ int TestSynthCallback(short *wav, int numsamples, espeak_EVENT *events)
 		else
 		if(type==espeakEVENT_PHONEME)
 		{
-			char buf[10];
+//			char buf[10];
 			fprintf(f_events,"[%s]\n",WordToString(events->id.number));  //old version, only 4 characters bytes
 //			memcpy(buf, events->id.string, 8);
 //			buf[8] = 0;
@@ -206,7 +206,7 @@ static const unsigned short KOI8_R[0x60] = {
 
 
 #define N_CHARS  34
-#define PH(c1,c2)  (c2<<8)+c1          // combine two characters into an integer for phoneme name 
+#define PH(c1,c2)  (c2<<8)+c1          // combine two characters into an integer for phoneme name
 
 
 
@@ -236,7 +236,7 @@ static void DecodePhonemes2(const char *inptr, char *outptr)
 			continue;     /* indicates unrecognised phoneme */
 		if((ph = phoneme_tab[phcode]) == NULL)
 			continue;
-	
+
 		if((ph->type == phSTRESS) && (ph->std_length <= 4) && (ph->program == 0))
 		{
 			if(ph->std_length > 2)
@@ -281,7 +281,7 @@ static void DecodePhonemes2(const char *inptr, char *outptr)
 			else
 			if(mnem != 0)
 			{
-				while((c = (mnem & 0xff)) != 0)	
+				while((c = (mnem & 0xff)) != 0)
 				{
 					*outptr++ = c;
 					mnem = mnem >> 8;
@@ -363,7 +363,7 @@ void Lexicon_It(int pass)
 	path_dir1 = wxFileName(fname_lex).GetPath();
 	strcpy(buf_out, path_dir1.mb_str(wxConvLocal));
 	sprintf(buf, "%s/IT_errors", buf_out);
-	
+
 	if((f_out = fopen(buf,"w")) == NULL)
 	{
 		str = wxString(buf, wxConvLocal);
@@ -412,7 +412,7 @@ void Lexicon_It(int pass)
 
 		if((p = strstr(buf,"//")) != NULL)
 			*p = 0;
-		
+
 		if((sscanf(buf,"%s %s",word,temp)) < 2)
 			continue;
 
@@ -464,7 +464,7 @@ void Lexicon_It(int pass)
 		}
 
 		// translate
-		TranslateWord(translator,&word1[1],0, NULL);
+		TranslateWord(translator,&word1[1],0, NULL, NULL);
 		DecodePhonemes(word_phonemes,phonemes);
 
 		stress_posn1 = 0;
@@ -487,7 +487,7 @@ void Lexicon_It(int pass)
 				vowel_ix++;
 		}
 
-		TranslateWord(translator,&word2[1],0, NULL);
+		TranslateWord(translator,&word2[1],0, NULL, NULL);
 		DecodePhonemes(word_phonemes,phonemes2);
 
 		vowel_ix = 1;
@@ -522,7 +522,7 @@ void Lexicon_It(int pass)
 						fputc(*p, f_listx);  // omit secondary stress marks
 				}
 				fputc('\n',f_listx);
-			
+
 			}
 			else
 			if((stress_posn1 != stress_posn2) && (stress_posn1 > 0) && (stress_posn2 > 0))
@@ -706,7 +706,7 @@ void Lexicon_De()
 		return;
 	}
 	path_dir1 = wxFileName(fname).GetPath();
-	
+
 	if((f_out = fopen("compare_de","w")) == NULL)
 	{
 		wxLogError(_T("Can't write file "));
@@ -802,7 +802,7 @@ void Lexicon_De()
 		}
 
 		// translate
-		TranslateWord(translator,&word2[1],0, NULL);
+		TranslateWord(translator,&word2[1],0, NULL, NULL);
 
 		DecodePhonemes2(word_phonemes,phonemes);  // also need to change some phoneme names
 
@@ -934,7 +934,7 @@ void Lexicon_Bg()
 
 	FILE *f_in;
 	FILE *f_out;
-	
+
 	char word[80];
 	char word_in[80];
 	char phonemes[N_WORD_PHONEMES];
@@ -1042,7 +1042,7 @@ void Lexicon_Bg()
 		*pw1 = 0;
 
 		// translate
-		TranslateWord(translator, &word[2],0, NULL);
+		TranslateWord(translator, &word[2],0, NULL, NULL);
 		DecodePhonemes(word_phonemes,phonemes);
 
 		// find the stress position in the translation
@@ -1101,7 +1101,7 @@ if(n_stress > 1) n_stress = 1;
 
 			if(done)
 				n_out++;
-		} 
+		}
 
 		n_words++;
 	}
@@ -1300,7 +1300,7 @@ void Lexicon_Ru()
 		}
 
 		// translate
-		TranslateWord(translator, &word2[1],0, NULL);
+		TranslateWord(translator, &word2[1],0, NULL, NULL);
 		DecodePhonemes(word_phonemes,phonemes);
 
 		// find the stress position in the translation
@@ -1383,10 +1383,10 @@ void Lexicon_Ru()
 				len = strlen(suffix);
 				if(len >= (wlen-2))
 					continue;
-	
+
 				if(ru_stress > (vcount - suffixes[sfx].syllables))
 					continue;
-				
+
 				if(strcmp(suffix,&word[wlen-len])==0)
 				{
 					strcpy(word2,word);
@@ -1547,7 +1547,7 @@ void CountWordFreq(wxString path, wcount **hashtab)
 			// skip leading spaces, numbers, etc
 			if(feof(f_in)) break;
 		}
-	
+
 		// read utf8 bytes until a space, number or punctuation
 		ix = 0;
 		while(!feof(f_in) && (c >= 'A') && (ix < sizeof(buf)-1))
@@ -1557,15 +1557,15 @@ void CountWordFreq(wxString path, wcount **hashtab)
 		}
 		buf[ix++] = 0;
 		buf[ix] = 0;
-	
+
 		// the buf may contain non-alphabetic characters
 		j = 0;
 		n_chars = 0;
 		for(k=0; k<ix; )
 		{
 			k += utf8_in(&wc,&buf[k]);
-			wc = towlower(wc);       // convert to lower case
-			if(iswalpha(wc))
+			wc = towlower2(wc);       // convert to lower case
+			if(iswalpha2(wc))
 			{
 				j += utf8_out(wc,&wbuf[j]);
 				n_chars++;
@@ -1583,7 +1583,7 @@ void CountWordFreq(wxString path, wcount **hashtab)
 		}
 	}
 	fclose(f_in);
-	
+
 }   // end of CountWordFreq
 
 
@@ -1645,7 +1645,7 @@ void MakeWordFreqList()
 		free(p);
 	}
 	fclose(f_out);
-	
+
 }  // end of Make WorkFreqList
 
 
@@ -1684,7 +1684,7 @@ void ConvertToUtf8()
 
 	while(!feof(f_in))
 	{
-		c = fgetc(f_in);
+		c = fgetc(f_in) & 0xff;
 		if(c >= 0xa0)
 			c = translator->charset_a0[c-0xa0];
 
@@ -1696,7 +1696,7 @@ void ConvertToUtf8()
 
 	wxLogStatus(_T("Written to: ")+fname+_T("_1"));
 
-}  // end of ConvertToItf8
+}  // end of ConvertToUtf8
 
 //******************************************************************************************************
 
@@ -2129,9 +2129,9 @@ void SetSpeedTab(void)
 	// Interpolation table to translate from words-per-minute to internal speed
 	// words-per-minute values (measured)
 	static float wpm1[N_WPM] =
-		{0, 82, 96, 108, 124, 134, 147, 162, 174, 189, 224, 259, 273, 289, 307, 326, 346, 361, 370 }; 
+		{0, 82, 96, 108, 124, 134, 147, 162, 174, 189, 224, 259, 273, 289, 307, 326, 346, 361, 370 };
 	// corresponding internal speed values
-	static float wpm2[N_WPM] = 
+	static float wpm2[N_WPM] =
 		{0,253,200, 170, 140, 125, 110,  95,  85,  75,  55,  40,  35,  30,  25,  20,  15,  10,  5 };
 
 	unsigned char speed_lookup[290];
@@ -2245,7 +2245,7 @@ void CharsetToUnicode(const char *charset)
 #ifdef deleted
 void Test2()
 {
-// 
+//
 	char buf[120];
 	FILE *f;
 	FILE *f_out;
@@ -2272,6 +2272,82 @@ void Test2()
 
 #endif
 
+#define MAX_WALPHA  0x24f
+void Make_walpha_tab()
+{//===================
+	int  ix;
+	int  value;
+	int  c;
+	short exceptions[40];
+	int  ex = 0;
+	FILE *f;
+
+	f = fopen("/home/jsd1/walpha_tab.txt","w");
+	for(ix=0x80; ix<=MAX_WALPHA; ix++)
+	{
+		value = 0;
+		if(iswalpha(ix))
+		{
+			value = 0xfe;  // no case
+			if(iswlower(ix))
+			{
+				value = 0xff;
+			}
+			else if(iswupper(ix))
+			{
+				c = towlower(ix);
+				if(c > 0)
+					value = c - ix;
+
+				if((value < 0) || (value > 0xfc))
+				{
+					exceptions[ex] = ix;
+					exceptions[ex+1] = c;
+					ex+=2;
+					value = 0xfd;
+				}
+			}
+
+		}
+
+		if(value > 0xfc)
+			fprintf(f," 0x%.2x,", value);
+		else
+			fprintf(f," %4d,", value);
+
+
+		if((ix % 16) == 15)
+		{
+			fprintf(f," // %.3x\n", ix & 0xfff0);
+		}
+
+	}
+
+	fprintf(f, "\nstatic const short wchar_tolower[] = {\n");
+
+	for(ix=0; ix<ex; ix+=2)
+	{
+		fprintf(f,"\t0x%.3x, 0x%.3x,\n", exceptions[ix], exceptions[ix+1]);
+	}
+	fprintf(f, "\t0,0 };\n");
+
+	fprintf(f, "\nstatic const short wchar_toupper[] = {\n");
+	for(ix=0x80; ix<=MAX_WALPHA; ix++)
+	{
+		if(iswlower(ix))
+		{
+			c = towupper(ix);
+			value =  ix - c;
+			if((value != 32) && (value != 1))
+			{
+				fprintf(f,"\t0x%.3x, 0x%.3x,\n", ix, c);
+			}
+		}
+	}
+	fprintf(f, "\t0,0 };\n");
+
+	fclose(f);
+}
 
 
 const char* text1 = "Hello world. Testing.";
@@ -2290,6 +2366,7 @@ void TestTest(int control)
 //CharsetToUnicode("ISO-8859-4");
 //CharsetToUnicode("ISCII");
 
+
 if(control==2)
 {
 	return;
@@ -2299,7 +2376,7 @@ if(control==2)
 	f = fopen("/home/jsd1/speechdata/text/test.txt","r");
 	if(f==NULL)
 		return;
-	
+
 
 	while(!feof(f) && (ix < sizeof(textbuf)-2))
 	{
@@ -2319,9 +2396,16 @@ if(control==2)
 	espeak_SetSynthCallback(TestSynthCallback);
 	espeak_SetUriCallback(TestUriCallback);
 
-	espeak_SetVoiceByName("en");
+	voice.languages = "fr";
+	espeak_SetVoiceByProperties(&voice);
+//	espeak_SetVoiceByName("fr");
+	espeak_Synth("1", 5, 0,  POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)user_data);
+
+//	voice.languages = "en";
+//	espeak_SetVoiceByProperties(&voice);
+	espeak_SetVoiceByName("de");
 	espeak_Synth(textbuf, strlen(textbuf)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)user_data);
-//  espeak_Synth(text1, strlen(text1)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)(user_data+1));
+//    espeak_Synth(text1, strlen(text1)+1, 0, POS_CHARACTER, 0,  espeakSSML|espeakCHARS_UTF8, &unique_identifier, (void *)(user_data+1));
 
   espeak_SetParameter(espeakPUNCTUATION, 1, 0);
   espeak_Synchronize();
