@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2014 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2015 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -35,7 +35,7 @@
 #include "translate.h"
 
 
-
+#define M_LIGATURE  0x8000
 #define M_NAME      0
 #define M_SMALLCAP  1
 #define M_TURNED    2
@@ -73,15 +73,15 @@ static int number_control;
 
 typedef struct {
 	const char *name;
-	int  flags;
+	int  accent_flags;   // bit 0, say before the letter name
 } ACCENTS;
 
 // these are tokens to look up in the *_list file.
 static ACCENTS accents_tab[] = {
 	{"_lig", 1},
-	{"_smc", 1},  // smallcap
-	{"_tur", 1},  // turned
-	{"_rev", 1},  // reversed
+	{"_smc", 0},  // smallcap
+	{"_tur", 0},  // turned
+	{"_rev", 0},  // reversed
 	{"_crl", 0},  // curl
 
 	{"_acu", 0},  // acute
@@ -107,7 +107,7 @@ static ACCENTS accents_tab[] = {
 
 #define CAPITAL  0
 #define LETTER(ch,mod1,mod2) (ch-59)+(mod1 << 6)+(mod2 << 11)
-#define LIGATURE(ch1,ch2,mod1) (ch1-59)+((ch2-59) << 6)+(mod1 << 12)+0x8000
+#define LIGATURE(ch1,ch2,mod1) (ch1-59)+((ch2-59) << 6)+(mod1 << 12)+M_LIGATURE
 
 
 #define L_ALPHA  60   // U+3B1
@@ -124,11 +124,12 @@ static ACCENTS accents_tab[] = {
 #define L_EZH     70 // U+292
 #define L_GLOTTAL 71 // U+294
 #define L_RTAP    72 // U+27E
+#define L_RLONG   73 // U+27C
 
 
 static const short non_ascii_tab[] = {
 	0, 0x3b1, 0x259, 0x25b, 0x3b3, 0x3b9, 0x153, 0x3c9,
-	0x3c6, 0x283, 0x3c5, 0x292, 0x294, 0x27e
+	0x3c6, 0x283, 0x3c5, 0x292, 0x294, 0x27e, 0x27c
 };
 
 
@@ -294,6 +295,8 @@ static const unsigned short letter_accents_0e0[] = {
 	CAPITAL,
 	LETTER('z',M_CARON,0),
 	LETTER('s',M_NAME,0), // long-s  // U+17f
+//	LETTER('b',M_STROKE,0),
+
 };
 
 
@@ -341,7 +344,7 @@ static const unsigned short letter_accents_250[] = {
 	0,//LETTER(L_OMEGA,M_CLOSED,0),
 	LETTER(L_PHI,0,0),		// U+278
 	LETTER('r',M_TURNED,0),
-	0,//LETTER('r',M_TURNED,M_LEG),
+	LETTER(L_RLONG,M_TURNED,0),
 	LETTER('r',M_RETROFLEX,M_TURNED),
 	0,//LETTER('r',M_LEG,0),
 	LETTER('r',M_RETROFLEX,0),
@@ -351,7 +354,7 @@ static const unsigned short letter_accents_250[] = {
 	LETTER('r',M_TURNED,M_SMALLCAP),
 	LETTER('s',M_RETROFLEX,0),
 	0,  // esh
-	0,//LETTER('j',M_BAR,L_IMPLOSIVE),
+	LETTER('j',M_HOOK,0), //LETTER('j',M_HOOK,M_BAR),
 	LETTER(L_ESH,M_REVERSED,0),
 	LETTER(L_ESH,M_CURL,0),
 	LETTER('t',M_TURNED,0),
@@ -419,6 +422,7 @@ void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf)
 	int accent_data = 0;
 	int accent1 = 0;
 	int accent2 = 0;
+	int flags1, flags2;
 	int basic_letter;
 	int letter2=0;
 	char ph_letter1[30];
@@ -443,7 +447,7 @@ void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf)
 		if(basic_letter < 'a')
 			basic_letter = non_ascii_tab[basic_letter-59];
 
-		if(accent_data & 0x8000)
+		if(accent_data & M_LIGATURE)
 		{
 			letter2 = (accent_data >> 6) & 0x3f;
 			letter2 += 59;
@@ -456,19 +460,25 @@ void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf)
 		}
 
 
-		if(Lookup(tr, accents_tab[accent1].name, ph_accent1) != 0)
+		if((accent1==0) && !(accent_data & M_LIGATURE))
+		{
+			// just a letter name, not an accented character or ligature
+			return;
+		}
+
+		if((flags1 = Lookup(tr, accents_tab[accent1].name, ph_accent1)) != 0)
 		{
 
 			if(LookupLetter2(tr, basic_letter, ph_letter1) != 0)
 			{
 				if(accent2 != 0)
 				{
-					if(Lookup(tr, accents_tab[accent2].name, ph_accent2) == 0)
+					if((flags2 = Lookup(tr, accents_tab[accent2].name, ph_accent2)) == 0)
 					{
 //						break;
 					}
 
-					if(accents_tab[accent2].flags & 1)
+					if(flags2 & FLAG_ACCENT_BEFORE)
 					{
 						strcpy(ph_buf,ph_accent2);
 						ph_buf += strlen(ph_buf);
@@ -485,7 +495,7 @@ void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf)
 				{
 					if(accent1 == 0)
 						strcpy(ph_buf, ph_letter1);
-					else if((tr->langopts.accents & 1) || (accents_tab[accent1].flags & 1))
+					else if((tr->langopts.accents & 1) || (flags1 & FLAG_ACCENT_BEFORE) || (accents_tab[accent1].accent_flags & 1))
 						sprintf(ph_buf,"%s%c%c%s", ph_accent1, phonPAUSE_VSHORT, phonSTRESS_P, ph_letter1);
 					else
 						sprintf(ph_buf,"%c%s%c%s%c", phonSTRESS_2, ph_letter1, phonPAUSE_VSHORT, ph_accent1, phonPAUSE_VSHORT);
@@ -1168,7 +1178,14 @@ int TranslateRoman(Translator *tr, char *word, char *ph_out, WORD_TAB *wtab)
 		return(0);    // not '2xx'
 
 	if(word[1] == ' ')
-		return(0);  // only one letter, don't speak as a Roman Number
+	{
+		if((tr->langopts.numbers & (NUM_ROMAN_CAPITALS | NUM_ROMAN_ORDINAL | NUM_ORDINAL_DOT)) && (wtab[0].flags & FLAG_HAS_DOT))
+		{
+			// allow single letter Roman ordinal followed by dot.
+		}
+		else
+			return(0);  // only one letter, don't speak as a Roman Number
+	}
 
 	word_start = word;
 	while((c = *word++) != ' ')
@@ -1547,7 +1564,7 @@ static int LookupNum2(Translator *tr, int value, int thousandplex, const int con
 
 		// no, speak as tens+units
 
-		if((control & 0x10) && (value < 10))
+		if((value < 10) && (control & 0x10))
 		{
 			// speak leading zero
 			Lookup(tr, "_0", ph_tens);
@@ -1684,7 +1701,18 @@ static int LookupNum2(Translator *tr, int value, int thousandplex, const int con
 						ph_tens[ix] = 0;
 				}
 			}
-			sprintf(ph_out,"%s%s%s",ph_tens, ph_digits, ph_ordinal);
+
+			if((tr->langopts.numbers2 & NUM2_ORDINAL_DROP_VOWEL) && (ph_ordinal[0] != 0))
+			{
+				ix = sprintf(ph_out,"%s%s", ph_tens, ph_digits);
+				if((ix > 0) && (phoneme_tab[(unsigned char)(ph_out[ix-1])]->type == phVOWEL))
+					ix--;
+				sprintf(&ph_out[ix], "%s", ph_ordinal);
+			}
+			else
+			{
+				sprintf(ph_out,"%s%s%s",ph_tens, ph_digits, ph_ordinal);
+			}
 		}
 	}
 
@@ -1951,6 +1979,12 @@ static int LookupNum3(Translator *tr, int value, char *ph_out, int suppress_null
 			x |= 0x208;  // use #f form for both tens and units
 		}
 
+		if((tr->langopts.numbers2 & NUM2_ZERO_TENS) && ((control & 1) || (hundreds > 0)))
+		{
+			// LANG=zh,
+			x |= 0x10;
+		}
+
 		if(LookupNum2(tr, tensunits, thousandplex, x | (control & 0x100), buf2) != 0)
 		{
 			if(tr->langopts.numbers & NUM_SINGLE_AND)
